@@ -26,6 +26,7 @@ WIDGET_RESIZE_MAXIMIZE = ('maximize', )
 
 
 default_font = soya.Font(os.path.join(soya.DATADIR, "FreeSans.ttf"), 25, 30)
+default_font.filename = "DEFAULT_FONT"
 
 # for font_filename in (
 #   # FreeFont
@@ -215,17 +216,14 @@ class Label(Widget):
 		Widget.resize(self, parent_left, parent_top, parent_width, parent_height)
 		
 	def __init__(self, master = None, text = "", align = 0, color = (1.0, 1.0, 1.0, 1.0), font = None, resize_style = None):
-		self._text    = text
-		self._color   = color
-		self._font    = font or default_font
-		self._align   = align # align: 0 left, 1 center
-		self._changed = -2
-		self._id      = glGenList()
+		self._text         = text
+		self._color        = color
+		self._font         = font or default_font
+		self._align        = align # align: 0 left, 1 center
+		self._changed      = -2
+		self._display_list = soya.DisplayList()
 		Widget.__init__(self, master, resize_style)
 		
-	def __del__(self):
-		glDeleteList(self._id)
-		
 	def build_display_list(self):
 		"""Label.build_display_list()
 
@@ -241,12 +239,12 @@ It might call glyph tessellation that would be included in the display list !"""
 		if self._changed != self._font._pixels_height:
 			self._font.create_glyphs(self._text)
 			
-			glNewList(self._id, GL_COMPILE_AND_EXECUTE)
+			glNewList(self._display_list.id, GL_COMPILE_AND_EXECUTE)
 			self.build_display_list()
 			glEndList()
 			self._changed = self._font._pixels_height
 		else:
-			glCallList(self._id)
+			glCallList(self._display_list.id)
 			
 
 
@@ -412,7 +410,7 @@ class Banner(Widget):
 			self.elements.append(BannerElement(t[1], v_color, v_font))
 		self.init()
 		
-	def advance_time(self, proportion):
+	def widget_advance_time(self, proportion):
 		self._position += proportion * self.speed
 		self.position = int(self._position)
 		if self.position >= self.total:
@@ -651,7 +649,7 @@ class ChoiceInput:
 
 class ChoiceList(Widget):
 
-	def __init__(self, master = None, choices = [], font = None, color = (1.0, 0.5, 0.5, 1.0), highlight = (1.0, 1.0, 0.0, 1.0), cancel = None,align=1):
+	def __init__(self, master = None, choices = [], font = None, color = (1.0, 0.5, 0.5, 1.0), highlight = (1.0, 1.0, 0.0, 1.0), cancel = None, align=1):
 		self.x_percent = 0.0
 		self.y_percent = 0.0
 		self.w_percent = 1.0
@@ -705,7 +703,7 @@ class ChoiceList(Widget):
 		else: self.choices[self.selected].key_down(key_id, mods)
 		
 	def mouse_move(self, x, y):
-		if (x >= self.left and x <= self.left + self.width):
+		if (x >= self.left) and (x <= self.left + self.width):
 			i = 0
 			nb = len(self.choices)
 			h1 = int (self.height / nb)
@@ -717,6 +715,8 @@ class ChoiceList(Widget):
 					break
 				t = t + h1
 				i = i + 1
+#			else: self.selected = -1
+#		else: self.selected = -1
 				
 	def render(self):
 		if self.visible:
